@@ -24,7 +24,7 @@ public:
   rtsRegionModelOptim(const rts::rtsRegionModelOptim<modeltype>& optim) : ModelOptim<modeltype>(optim.model, optim.matrix, optim.re), region(optim.region) {};
   
   void        update_theta(const dblvec &theta) override;
-  void        update_u(const MatrixXd& u) override;
+  void        update_u(const MatrixXd& u, bool append) override;
   void        update_rho(double rho);
   template<class algo, typename = std::enable_if_t<std::is_base_of<optim_algo, algo>::value> >
   void        ml_beta();
@@ -36,7 +36,7 @@ public:
   double      log_likelihood_rho(const dblvec &rho);
   double      log_likelihood_rho_with_gradient(const VectorXd &rho, VectorXd& g);
   double      log_likelihood_beta(const dblvec &beta);
-  double      log_likelihood() override;
+  double      log_likelihood(bool beta) override;
   double      full_log_likelihood() override;
   ArrayXXd    region_intensity(bool uselog = true);
   ArrayXXd    y_predicted(bool uselog = true);
@@ -50,6 +50,12 @@ template<class algo, typename>
 inline void rts::rtsRegionModelOptim<modeltype>::ml_beta()
 {  
   dblvec start = this->get_start_values(true,false,false);
+
+  this->previous_ll_values.first = this->current_ll_values.first;
+  if(this->ll_previous.rows() != this->ll_current.rows()) this->ll_previous.resize(this->ll_current.rows(),NoChange);
+  double old_ll = this->log_likelihood_beta(start);
+  this->ll_previous.col(0) = this->ll_current.col(0);
+
   if constexpr (std::is_same_v<algo,LBFGS>)
   {
     throw std::runtime_error("L-BGFS not available with regional data model yet.");
@@ -80,6 +86,9 @@ inline void rts::rtsRegionModelOptim<modeltype>::ml_beta()
     }
     op.minimise();
   }
+
+  int eval_size = this->control.saem ? this->re.mcmc_block_size : this->ll_current.rows();
+  this->current_ll_values.first = this->ll_current.col(0).tail(eval_size).mean();
 }
 
 template<typename modeltype>
@@ -88,14 +97,15 @@ inline void rts::rtsRegionModelOptim<modeltype>::ml_theta(){
   dblvec start = this->get_start_values(false,true,false);  
   dblvec lower = this->get_lower_values(false,true,false);
   dblvec upper = this->get_upper_values(false,true,false);
-  // if(this->model.covariance.grid.T > 1)
-  // {
-  //   start.push_back(this->model.covariance.rho);
-  //   lower.push_back(-1);
-  //   upper.push_back(2);
-  // }
-  if(this->re.scaled_u_.cols() != this->re.u_.cols())this->re.scaled_u_.conservativeResize(NoChange,this->re.u_.cols());
+
+   if(this->re.scaled_u_.cols() != this->re.u_.cols())this->re.scaled_u_.resize(NoChange,this->re.u_.cols());
   this->re.scaled_u_ = this->model.covariance.Lu(this->re.u_);  
+
+  if(this->ll_previous.rows() != this->ll_current.rows()) this->ll_previous.resize(this->ll_current.rows(),NoChange);
+  this->previous_ll_values.second = this->current_ll_values.second;
+  double old_ll = this->log_likelihood_theta(start);
+  this->ll_previous.col(1) = this->ll_current.col(1);
+
   if constexpr (std::is_same_v<algo,LBFGS>){
     VectorXd start_vec = Map<VectorXd>(start.data(),start.size());
     optim<double(const VectorXd&, VectorXd&),algo> op(start_vec); 
@@ -141,6 +151,12 @@ inline void rts::rtsRegionModelOptim<modeltype>::ml_theta(){
     op.minimise();
   }
   this->re.zu_ = this->model.covariance.ZLu(this->re.u_);
+
+  if(this->model.covariance.grid.T == 1)
+  {
+    int eval_size = this->control.saem ? this->re.mcmc_block_size : this->ll_current.rows();
+    this->current_ll_values.second = this->ll_current.col(1).tail(eval_size).mean();
+  }  
 }
 
 template<typename modeltype>
@@ -198,46 +214,95 @@ inline void rts::rtsRegionModelOptim<modeltype>::ml_rho()
     op.minimise();
   }
   this->re.zu_ = this->model.covariance.ZLu(this->re.u_);
+
+  int eval_size = this->control.saem ? this->re.mcmc_block_size : this->ll_current.rows();
+  this->current_ll_values.second = this->ll_current.col(1).tail(eval_size).mean();
 }
 
 template<typename modeltype>
 inline double rts::rtsRegionModelOptim<modeltype>::log_likelihood_theta(const dblvec& theta)
 {
-    // if(this->model.covariance.grid.T == 1)
-    // {
-    //   this->model.covariance.update_parameters(theta);
-    // } else {
-    //   dblvec theta_p(2);
-    //   theta_p[0] = theta[0];
-    //   theta_p[1] = theta[1];
-    //   this->model.covariance.update_parameters(theta_p);
-    //   this->model.covariance.update_rho(theta[2]);
-    // }
-    this->model.covariance.update_parameters(theta);
-    double logl = 0;
-  #pragma omp parallel for reduction (+:logl)
-    for(int i = 0; i < this->re.scaled_u_.cols(); i++)
-    {
-      logl += this->model.covariance.log_likelihood(this->re.scaled_u_.col(i));
+  this->model.covariance.update_parameters(theta);
+  this->fn_counter.second += this->Q() * this->re.scaled_u_.cols();
+#pragma omp parallel
+  for(int i = 0; i < this->re.scaled_u_.cols(); i++)
+  {
+    this->ll_current(i,1) = this->model.covariance.log_likelihood(this->re.scaled_u_.col(i));
+  }
+  double ll = 0;
+  if(this->control.saem)
+  {
+    int     iteration = std::max((int)this->re.zu_.cols() / this->re.mcmc_block_size, 1);
+    double  gamma = pow(1.0/iteration,this->control.alpha);
+    double  ll_t = 0;
+    double  ll_pr = 0;
+    for(int i = 0; i < iteration; i++){
+      int lower_range = i * this->re.mcmc_block_size;
+      int upper_range = (i + 1) * this->re.mcmc_block_size;
+      if(i == (iteration - 1) && iteration > 1){
+        double ll_t_c = ll_t;
+        double ll_pr_c = ll_pr;
+        ll_t = ll_t + gamma*(this->ll_current.col(1).segment(lower_range, this->re.mcmc_block_size).mean() - ll_t);
+        if(this->control.pr_average) ll_pr += ll_t;
+        for(int j = lower_range; j < upper_range; j++)
+        {
+          this->ll_current(j,1) = ll_t_c + gamma*(this->ll_current(j,1) - ll_t_c);
+          if(this->control.pr_average) this->ll_current(j,1) = (this->ll_current(j,1) + ll_pr_c)/((double)iteration);
+        }
+      } else {
+        ll_t = ll_t + gamma*(this->ll_current.col(1).segment(lower_range, this->re.mcmc_block_size).mean() - ll_t);
+        if(this->control.pr_average) ll_pr += ll_t;
+      }
     }
-    return -1*logl/this->re.u_.cols();
+    if(this->control.pr_average){
+      ll = ll_pr / (double)iteration;
+    } else {
+      ll = ll_t;
+    }
+  } else {
+    ll = this->ll_current.col(1).mean();
+  }
+  return -1*ll;
 }
 
 template<>
 inline double rts::rtsRegionModelOptim<BitsHSGP>::log_likelihood_theta(const dblvec& theta){
-  // if(this->model.covariance.grid.T == 1)
-  //   {
-  //     this->model.covariance.update_parameters(theta);
-  //   } else {
-  //     dblvec theta_p(2);
-  //     theta_p[0] = theta[0];
-  //     theta_p[1] = theta[1];
-  //     this->model.covariance.update_parameters(theta_p);
-  //     this->model.covariance.update_rho(theta[2]);
-  //   }
   this->model.covariance.update_parameters(theta);
   this->re.zu_ = this->model.covariance.ZLu(this->re.u_);
-  double ll = this->log_likelihood();
+  double ll = this->log_likelihood(false);
+  this->fn_counter.first += this->model.n() * this->re.scaled_u_.cols();
+  if(this->control.saem)
+  {
+    int     iteration = std::max((int)this->re.zu_.cols() / this->re.mcmc_block_size, 1);
+    double  gamma = pow(1.0/iteration,this->control.alpha);
+    double  ll_t = 0;
+    double  ll_pr = 0;
+    for(int i = 0; i < iteration; i++){
+      int lower_range = i * this->re.mcmc_block_size;
+      int upper_range = (i + 1) * this->re.mcmc_block_size;
+      if(i == (iteration - 1) && iteration > 1){
+        double ll_t_c = ll_t;
+        double ll_pr_c = ll_pr;
+        ll_t = ll_t + gamma*(this->ll_current.col(0).segment(lower_range, this->re.mcmc_block_size).mean() - ll_t);
+        if(this->control.pr_average) ll_pr += ll_t;
+        for(int j = lower_range; j < upper_range; j++)
+        {
+          this->ll_current(j,0) = ll_t_c + gamma*(this->ll_current(j,0) - ll_t_c);
+          if(this->control.pr_average) this->ll_current(j,0) = (this->ll_current(j,0) + ll_pr_c)/((double)iteration);
+        }
+      } else {
+        ll_t = ll_t + gamma*(this->ll_current.col(0).segment(lower_range, this->re.mcmc_block_size).mean() - ll_t);
+        if(this->control.pr_average) ll_pr += ll_t;
+      }
+    }
+    if(this->control.pr_average){
+      ll = ll_pr / (double)iteration;
+    } else {
+      ll = ll_t;
+    }
+  } else {
+    ll = this->log_likelihood(false);
+  }
   return -1*ll;
 }
 
@@ -245,69 +310,228 @@ template<>
 inline double rts::rtsRegionModelOptim<BitsHSGPRegion>::log_likelihood_theta(const dblvec& theta){
   this->model.covariance.update_parameters(theta);
   this->re.zu_ = this->model.covariance.ZLu(this->re.u_);
-  // if(this->model.covariance.grid.T == 1)
-  //   {
-  //     this->model.covariance.update_parameters(theta);
-  //   } else {
-  //     dblvec theta_p(2);
-  //     theta_p[0] = theta[0];
-  //     theta_p[1] = theta[1];
-  //     this->model.covariance.update_parameters(theta_p);
-  //     this->model.covariance.update_rho(theta[2]);
-  //   }
-  double ll = this->log_likelihood();
+  double ll = this->log_likelihood(false);
+  this->fn_counter.first += this->model.n() * this->re.scaled_u_.cols();
+  if(this->control.saem)
+  {
+    int     iteration = std::max((int)this->re.zu_.cols() / this->re.mcmc_block_size, 1);
+    double  gamma = pow(1.0/iteration,this->control.alpha);
+    double  ll_t = 0;
+    double  ll_pr = 0;
+    for(int i = 0; i < iteration; i++){
+      int lower_range = i * this->re.mcmc_block_size;
+      int upper_range = (i + 1) * this->re.mcmc_block_size;
+      if(i == (iteration - 1) && iteration > 1){
+        double ll_t_c = ll_t;
+        double ll_pr_c = ll_pr;
+        ll_t = ll_t + gamma*(this->ll_current.col(0).segment(lower_range, this->re.mcmc_block_size).mean() - ll_t);
+        if(this->control.pr_average) ll_pr += ll_t;
+        for(int j = lower_range; j < upper_range; j++)
+        {
+          this->ll_current(j,0) = ll_t_c + gamma*(this->ll_current(j,0) - ll_t_c);
+          if(this->control.pr_average) this->ll_current(j,0) = (this->ll_current(j,0) + ll_pr_c)/((double)iteration);
+        }
+      } else {
+        ll_t = ll_t + gamma*(this->ll_current.col(0).segment(lower_range, this->re.mcmc_block_size).mean() - ll_t);
+        if(this->control.pr_average) ll_pr += ll_t;
+      }
+    }
+    if(this->control.pr_average){
+      ll = ll_pr / (double)iteration;
+    } else {
+      ll = ll_t;
+    }
+  } else {
+    ll = this->log_likelihood(false);
+  }
   return -1*ll;
 }
 
 template<typename modeltype>
 inline double rts::rtsRegionModelOptim<modeltype>::log_likelihood_rho(const dblvec& rho){
   this->model.covariance.update_rho(rho[0]);
-  double logl = 0;
-  #pragma omp parallel for reduction (+:logl)
-    for(int i = 0; i < this->re.scaled_u_.cols(); i++)
-    {
-      logl += this->model.covariance.log_likelihood(this->re.scaled_u_.col(i));
+  this->fn_counter.second += this->Q() * this->re.scaled_u_.cols();
+#pragma omp parallel
+  for(int i = 0; i < this->re.scaled_u_.cols(); i++)
+  {
+    this->ll_current(i,1) = this->model.covariance.log_likelihood(this->re.scaled_u_.col(i));
+  }
+  double ll = 0;
+  if(this->control.saem)
+  {
+    int     iteration = std::max((int)this->re.zu_.cols() / this->re.mcmc_block_size, 1);
+    double  gamma = pow(1.0/iteration,this->control.alpha);
+    double  ll_t = 0;
+    double  ll_pr = 0;
+    for(int i = 0; i < iteration; i++){
+      int lower_range = i * this->re.mcmc_block_size;
+      int upper_range = (i + 1) * this->re.mcmc_block_size;
+      if(i == (iteration - 1) && iteration > 1){
+        double ll_t_c = ll_t;
+        double ll_pr_c = ll_pr;
+        ll_t = ll_t + gamma*(this->ll_current.col(1).segment(lower_range, this->re.mcmc_block_size).mean() - ll_t);
+        if(this->control.pr_average) ll_pr += ll_t;
+        for(int j = lower_range; j < upper_range; j++)
+        {
+          this->ll_current(j,1) = ll_t_c + gamma*(this->ll_current(j,1) - ll_t_c);
+          if(this->control.pr_average) this->ll_current(j,1) = (this->ll_current(j,1) + ll_pr_c)/((double)iteration);
+        }
+      } else {
+        ll_t = ll_t + gamma*(this->ll_current.col(1).segment(lower_range, this->re.mcmc_block_size).mean() - ll_t);
+        if(this->control.pr_average) ll_pr += ll_t;
+      }
     }
-  return -1*logl;
+    if(this->control.pr_average){
+      ll = ll_pr / (double)iteration;
+    } else {
+      ll = ll_t;
+    }
+  } else {
+    ll = this->ll_current.col(1).mean();
+  }
+  return -1*ll;
 }
 
 template<>
 inline double rts::rtsRegionModelOptim<BitsHSGP>::log_likelihood_rho(const dblvec& rho){
   this->model.covariance.update_rho(rho[0]);
   this->re.zu_ = this->model.covariance.ZLu(this->re.u_);
-  double logl = log_likelihood();
-  return -1*logl;
+  double ll = this->log_likelihood(false);
+  this->fn_counter.first += this->model.n() * this->re.scaled_u_.cols();
+  if(this->control.saem)
+  {
+    int     iteration = std::max((int)this->re.zu_.cols() / this->re.mcmc_block_size, 1);
+    double  gamma = pow(1.0/iteration,this->control.alpha);
+    double  ll_t = 0;
+    double  ll_pr = 0;
+    for(int i = 0; i < iteration; i++){
+      int lower_range = i * this->re.mcmc_block_size;
+      int upper_range = (i + 1) * this->re.mcmc_block_size;
+      if(i == (iteration - 1) && iteration > 1){
+        double ll_t_c = ll_t;
+        double ll_pr_c = ll_pr;
+        ll_t = ll_t + gamma*(this->ll_current.col(0).segment(lower_range, this->re.mcmc_block_size).mean() - ll_t);
+        if(this->control.pr_average) ll_pr += ll_t;
+        for(int j = lower_range; j < upper_range; j++)
+        {
+          this->ll_current(j,0) = ll_t_c + gamma*(this->ll_current(j,0) - ll_t_c);
+          if(this->control.pr_average) this->ll_current(j,0) = (this->ll_current(j,0) + ll_pr_c)/((double)iteration);
+        }
+      } else {
+        ll_t = ll_t + gamma*(this->ll_current.col(0).segment(lower_range, this->re.mcmc_block_size).mean() - ll_t);
+        if(this->control.pr_average) ll_pr += ll_t;
+      }
+    }
+    if(this->control.pr_average){
+      ll = ll_pr / (double)iteration;
+    } else {
+      ll = ll_t;
+    }
+  } else {
+    ll = this->log_likelihood(false);
+  }
+  return -1*ll;
 }
 
 template<>
 inline double rts::rtsRegionModelOptim<BitsHSGPRegion>::log_likelihood_rho(const dblvec& rho){
   this->model.covariance.update_rho(rho[0]);
   this->re.zu_ = this->model.covariance.ZLu(this->re.u_);
-  double logl = log_likelihood();
-  return -1*logl;
+  double ll = log_likelihood(false);
+  this->fn_counter.first += this->model.n() * this->re.scaled_u_.cols();
+  if(this->control.saem)
+  {
+    int     iteration = std::max((int)this->re.zu_.cols() / this->re.mcmc_block_size, 1);
+    double  gamma = pow(1.0/iteration,this->control.alpha);
+    double  ll_t = 0;
+    double  ll_pr = 0;
+    for(int i = 0; i < iteration; i++){
+      int lower_range = i * this->re.mcmc_block_size;
+      int upper_range = (i + 1) * this->re.mcmc_block_size;
+      if(i == (iteration - 1) && iteration > 1){
+        double ll_t_c = ll_t;
+        double ll_pr_c = ll_pr;
+        ll_t = ll_t + gamma*(this->ll_current.col(0).segment(lower_range, this->re.mcmc_block_size).mean() - ll_t);
+        if(this->control.pr_average) ll_pr += ll_t;
+        for(int j = lower_range; j < upper_range; j++)
+        {
+          this->ll_current(j,0) = ll_t_c + gamma*(this->ll_current(j,0) - ll_t_c);
+          if(this->control.pr_average) this->ll_current(j,0) = (this->ll_current(j,0) + ll_pr_c)/((double)iteration);
+        }
+      } else {
+        ll_t = ll_t + gamma*(this->ll_current.col(0).segment(lower_range, this->re.mcmc_block_size).mean() - ll_t);
+        if(this->control.pr_average) ll_pr += ll_t;
+      }
+    }
+    if(this->control.pr_average){
+      ll = ll_pr / (double)iteration;
+    } else {
+      ll = ll_t;
+    }
+  } else {
+    ll = this->log_likelihood(false);
+  }
+  return -1*ll;
 }
 
 template<typename modeltype>
 inline double rts::rtsRegionModelOptim<modeltype>::log_likelihood_rho_with_gradient(const VectorXd& rho, VectorXd& g)
 {
-  this->model.covariance.update_rho(rho(0));
-  // update_rho(rho(0));
-  double logl = 0;
-  #pragma omp parallel for reduction (+:logl)
-  for(int i = 0; i < this->re.scaled_u_.cols(); i++)
-    {
-      logl += this->model.covariance.log_likelihood(this->re.scaled_u_.col(i));
-    }
-  g = this->model.covariance.log_gradient_rho(this->re.scaled_u_);
-  g.array() *= -1.0;
-  return -1*logl;
+  if(this->control.saem){
+    throw std::runtime_error("L-BFGS-B not available with SAEM");
+  } else {
+    this->model.covariance.update_rho(rho(0));
+    // update_rho(rho(0));
+    double logl = 0;
+    #pragma omp parallel for reduction (+:logl)
+    for(int i = 0; i < this->re.scaled_u_.cols(); i++)
+      {
+        logl += this->model.covariance.log_likelihood(this->re.scaled_u_.col(i));
+      }
+    g = this->model.covariance.log_gradient_rho(this->re.scaled_u_);
+    g.array() *= -1.0;
+    return -1*logl;
+  }
 }
 
 template<typename modeltype>
 inline double rts::rtsRegionModelOptim<modeltype>::log_likelihood_beta(const dblvec& beta)
 {
   this->model.linear_predictor.update_parameters(beta);
-  double ll = this->log_likelihood();
+  double ll = this->log_likelihood(true);
+  this->fn_counter.first += this->model.n() * this->re.scaled_u_.cols();
+  if(this->control.saem)
+  {
+    int     iteration = std::max((int)this->re.zu_.cols() / this->re.mcmc_block_size, 1);
+    double  gamma = pow(1.0/iteration,this->control.alpha);
+    double  ll_t = 0;
+    double  ll_pr = 0;
+    for(int i = 0; i < iteration; i++){
+      int lower_range = i * this->re.mcmc_block_size;
+      int upper_range = (i + 1) * this->re.mcmc_block_size;
+      if(i == (iteration - 1) && iteration > 1){
+        double ll_t_c = ll_t;
+        double ll_pr_c = ll_pr;
+        ll_t = ll_t + gamma*(this->ll_current.col(0).segment(lower_range, this->re.mcmc_block_size).mean() - ll_t);
+        if(this->control.pr_average) ll_pr += ll_t;
+        for(int j = lower_range; j < upper_range; j++)
+        {
+          this->ll_current(j,0) = ll_t_c + gamma*(this->ll_current(j,0) - ll_t_c);
+          if(this->control.pr_average) this->ll_current(j,0) = (this->ll_current(j,0) + ll_pr_c)/((double)iteration);
+        }
+      } else {
+        ll_t = ll_t + gamma*(this->ll_current.col(0).segment(lower_range, this->re.mcmc_block_size).mean() - ll_t);
+        if(this->control.pr_average) ll_pr += ll_t;
+      }
+    }
+    if(this->control.pr_average){
+      ll = ll_pr / (double)iteration;
+    } else {
+      ll = ll_t;
+    }
+  } else {
+    ll = this->log_likelihood(true);
+  }
   return -1*ll;
 }
 
@@ -326,43 +550,56 @@ inline void rts::rtsRegionModelOptim<modeltype>::update_rho(double rho)
 }
 
 template<typename modeltype>
-inline void rts::rtsRegionModelOptim<modeltype>::update_u(const MatrixXd& u_)
+inline void rts::rtsRegionModelOptim<modeltype>::update_u(const MatrixXd& u_, bool append)
 {
-  if(u_.cols()!=this->re.u(false).cols())
-  {
-    this->re.u_.conservativeResize(this->model.covariance.Q(),u_.cols());
-    this->re.zu_.resize(this->model.covariance.Q(),u_.cols());
+  int newcolsize = u_.cols();
+  int currcolsize = this->re.u_.cols();
+
+  if(append){
+    this->re.u_.conservativeResize(NoChange,currcolsize + newcolsize);
+    this->re.zu_.conservativeResize(NoChange,currcolsize + newcolsize);
+    this->re.u_.rightCols(newcolsize) = u_;
+    this->ll_current.resize(currcolsize + newcolsize,NoChange);
+  } else {
+    if(u_.cols()!=this->re.u_.cols()){
+      this->re.u_.resize(NoChange,newcolsize);
+      this->re.zu_.resize(NoChange,newcolsize);
+      this->re.u_ = u_;
+      if(newcolsize != this->ll_current.rows()) this->ll_current.resize(newcolsize,NoChange);
+    }
   }
-  this->re.u_ = u_;
   this->re.zu_ = this->model.covariance.ZLu(this->re.u_);
 }
 
 template<typename modeltype>
-inline double rts::rtsRegionModelOptim<modeltype>::log_likelihood()
+inline double rts::rtsRegionModelOptim<modeltype>::log_likelihood(bool beta)
 {
-  double ll = 0;
+  
   ArrayXXd xb = y_predicted(true);
+  int llcol = beta ? 0 : 1;
+  this->ll_current.col(llcol).setZero();
+
   if(this->model.weighted){
-#pragma omp parallel for reduction (+:ll) collapse(2)
     for(int j=0; j<xb.cols() ; j++)
     {
+#pragma omp parallel for
       for(int i = 0; i<xb.rows(); i++)
       {
-        ll += this->model.data.weights(i)*glmmr::maths::log_likelihood(this->model.data.y(i),xb(i,j),this->model.data.variance(i),this->model.family.family,this->model.family.link);
+        this->ll_current(j,llcol) += this->model.data.weights(i)*glmmr::maths::log_likelihood(this->model.data.y(i),xb(i,j),this->model.data.variance(i),this->model.family.family,this->model.family.link);
       }
     }
-    ll *= this->model.data.weights.sum()/this->model.n();
+    this->ll_current.col(llcol) *= this->model.data.weights.sum()/this->model.n();
   } else {
 #pragma omp parallel for reduction (+:ll) collapse(2)
   for(int j=0; j<xb.cols() ; j++)
   {
     for(int i = 0; i< xb.rows(); i++)
     {
-      ll += glmmr::maths::log_likelihood(this->model.data.y(i),xb(i,j),this->model.data.variance(i),this->model.family.family,this->model.family.link);
+      this->ll_current(j,llcol) += glmmr::maths::log_likelihood(this->model.data.y(i),xb(i,j),this->model.data.variance(i),this->model.family.family,this->model.family.link);
     }
   }
 }
-  return ll/xb.cols();
+  return this->ll_current.col(llcol).mean();
 }
 
 template<typename modeltype>
@@ -397,7 +634,7 @@ inline ArrayXXd rts::rtsRegionModelOptim<modeltype>::region_intensity(bool uselo
 template<typename modeltype>
 inline double rts::rtsRegionModelOptim<modeltype>::full_log_likelihood()
 {
-  double ll = rts::rtsRegionModelOptim<modeltype>::log_likelihood();
+  double ll = rts::rtsRegionModelOptim<modeltype>::log_likelihood(true);
   double logl = 0;
   MatrixXd Lu = this->model.covariance.Lu(this->re.u_);
 #pragma omp parallel for reduction (+:logl)
